<?php

/**
 * @version    CVS: 1.0.0
 * @package    Com_Resourcespace
 * @author     Resource Space <shoaibalam.augurs@gmail.com>
 * @copyright  2025 ResourceSpace
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access
defined('_JEXEC') or die;

use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Http\Exception\HttpException;
use Joomla\CMS\Http\Http;
use \Joomla\CMS\Factory;
use \Joomla\CMS\Language\Text;

/**
 * Resourcespace helper.
 *
 * @since  1.6
 */
class ResourcespaceHelper
{
    /**
     * Configure the Linkbar.
     *
     * @param   string  $vName  string
     *
     * @return void
     */
    public static function addSubmenu($vName = '')
    {
        JHtmlSidebar::addEntry(
            Text::_('COM_RESOURCESPACE_TITLE_RESOURCESPACES'),
            'index.php?option=com_resourcespace&view=resourcespaces',
            $vName == 'resourcespaces'
        );
    }

    /**
     * Gets the files attached to an item
     *
     * @param   int     $pk     The item's id
     *
     * @param   string  $table  The table's name
     *
     * @param   string  $field  The field's name
     *
     * @return  array  The files
     */
    public static function getFiles($pk, $table, $field)
    {
        $db    = Factory::getDbo();
        $query = $db->getQuery(true);

        $query
            ->select($field)
            ->from($table)
            ->where('id = ' . (int) $pk);

        $db->setQuery($query);

        return explode(',', $db->loadResult());
    }

    /**
     * Gets a list of the actions that can be performed.
     *
     * @return    JObject
     *
     * @since    1.6
     */
    public static function getActions()
    {
        $user   = Factory::getUser();
        $result = new JObject;

        $assetName = 'com_resourcespace';

        $actions = [
            'core.admin', 'core.manage', 'core.create', 'core.edit', 'core.edit.own', 'core.edit.state', 'core.delete',
        ];

        foreach ($actions as $action) {
            $result->set($action, $user->authorise($action, $assetName));
        }

        return $result;
    }

    public static function getConfig()
    {
        // Get component params
        $params = ComponentHelper::getParams('com_resourcespace');

        // Retrieve stored values
        $apiUrl   = trim($params->get('resourcespace_api_url', ''));
        $username = trim($params->get('resourcespace_username', ''));
        $apiKey   = trim($params->get('resourcespace_api_key', ''));

        // Validate if all required values are set
        if (empty($apiUrl) || empty($username) || empty($apiKey)) {
            return ['error' => 'No API key added. Please configure the ResourceSpace settings.'];
        }

        return [
            'apiUrl'   => $apiUrl,
            'username' => $username,
            'apiKey'   => $apiKey,
        ];
    }
    /**
     * Makes an API request to the given URL using Joomla's HTTP client.
     *
     * @param   string  $url  The API endpoint URL.
     *
     * @return  mixed   The API response body or null in case of failure.
     */
    public static function makeApiRequest($url)
    {
        try {
            // Get Joomla's HTTP client (Http::getInstance() does not exist).
            $httpClient = new Http();

            // Make the GET request.
            $response = $httpClient->get($url, [
                'timeout' => 40,
                'verify'  => false, // Disabling SSL verification (use with caution).
            ]);

            // Return the response body.
            return $response->getBody();
        } catch (HttpException $e) {
            // Log the error in Joomla's log system.
            Factory::getApplication()->enqueueMessage(Text::sprintf('API request failed: %s', $e->getMessage()), 'error');
            return null;
        }
    }

    public static function getResources($offset, $limit, $search = "")
    {
        // Load configuration.
        $config   = self::getConfig();
        $apiUrl   = $config['apiUrl'];
        $username = $config['username'];
        $apiKey   = $config['apiKey'];

        if (! $apiUrl || ! $username || ! $apiKey) {
            return [];
        }

        $searchTerm = $search;

        $query = "user={$username}&function=do_search&fetchrows={$offset},{$limit}&search=" . urlencode($searchTerm);

        // Generate the signature.
        $signature = hash('sha256', $apiKey . $query);

        // Full API URL with the signature.
        $fullUrl = "{$apiUrl}?{$query}&sign={$signature}";

        // Make the API request.
        $response = self::makeApiRequest($fullUrl);

        // Decode the JSON response.
        $resources = $response;

        if ($resources && is_array($resources)) {
            // foreach ($resources as $resource) {
            //     $this->createMediaEntity($resource);
            // }
        }
        return $resources;
    }

    public static function getResoucePath($ref_id, $extension)
    {

        // Load configuration.
        $config   = self::getConfig();
        $apiUrl   = $config['apiUrl'];
        $username = $config['username'];
        $apiKey   = $config['apiKey'];

        if (! $apiUrl || ! $username || ! $apiKey) {
            return [];
        }

        $query = "user={$username}&function=get_resource_path&external=true&extension={$extension}&ref=" . urlencode($ref_id);

        // Generate the signature.
        $signature = hash('sha256', $apiKey . $query);

        // Full API URL with the signature.
        $fullUrl = "{$apiUrl}?{$query}&sign={$signature}";

        // Make the API request.
        $response = self::makeApiRequest($fullUrl);

        // Decode the JSON response.
        $resources = json_decode($response, true);

        return $resources;
    }

    public static function fetchSearchResults($query)
    {
        // Get the API configuration
        $config = self::getConfig();

        // Check for an error in configuration
        if (isset($config['error'])) {
            return $config['error'];
        }

        $response = self::getResources(0, 10, $query);
        $data     = json_decode($response, true)['data'];
        $results  = [];
        if ($response && is_array($data)) {
            foreach ($data as $resource) {
                $imageUrl      = self::getResoucePath($resource['ref'], $resource['file_extension']);
                $last_modified = $resource['file_modified'];
                $results[]     = [
                    'thumbnail'   => $imageUrl,
                    'title'       => $resource['field8'],
                    'ref_id'      => $resource['ref'],
                    'extension'   => $resource['file_extension'],
                    'modified_at' => $last_modified,
                ];
            }
        }

        // Check for errors
        if (! $response) {
            return 'API request failed. Please check your API URL and credentials.';
        }

        return $results;
    }

    public static function getResourcesCron()
    {
        // Load configuration.
        $config   = self::getConfig();
        $apiUrl   = $config['apiUrl'];
        $username = $config['username'];
        $apiKey   = $config['apiKey'];

        if (! $apiUrl || ! $username || ! $apiKey) {
            return [];
        }

        $searchTerm = $search;

        $query = "user={$username}&function=do_search&search=" . urlencode($searchTerm);

        // Generate the signature.
        $signature = hash('sha256', $apiKey . $query);

        // Full API URL with the signature.
        $fullUrl = "{$apiUrl}?{$query}&sign={$signature}";

        // Make the API request.
        $response = self::makeApiRequest($fullUrl);

        // Decode the JSON response.
        $resources = $response;

        if ($resources && is_array($resources)) {
            $ref_ids = [];
            foreach ($resources as $resource) {
                $ref_ids[] = $resource['ref'];
            }
        }
        return $resources;
    }
    public static function fetchForCronJob($query)
    {
        // Get the API configuration
        $config = self::getConfig();

        // Check for an error in configuration
        if (isset($config['error'])) {
            return $config['error'];
        }

        $response = self::getResourcesCron();
        $data     = json_decode($response, true);
        $results  = [];
        $ref_ids = [];
        if ($response && is_array($data)) {
            foreach ($data as $resource) {
                $imageUrl      = self::getResoucePath($resource['ref'], $resource['file_extension']);
                $last_modified = $resource['file_modified'];
                if ($imageUrl != null) {
                    self::saveImageFromUrl($imageUrl, $resource['field8'], $resource['ref'], $resource['file_extension']);
                }
                $ref_ids[] = $resource['ref'];
            }
        }

        // Check for errors
        if (! $response) {
            return 'API request failed. Please check your API URL and credentials.';
        }

        if(!empty($ref_ids)){
            self::deleteMissingMedia($ref_ids);
        }

        return $results;
    }

    private static function deleteMissingMedia($externalRefIds)
    {
        $db = JFactory::getDbo();

        // Get all media `ref_id`s from the database
        $query = $db->getQuery(true)
            ->select($db->quoteName(['id', 'ref_id', 'media_id', 'image_path']))
            ->from($db->quoteName('#__resourcespace_media'));
        $db->setQuery($query);
        $allMedia = $db->loadObjectList();

        $mediaToDelete  = [];
        $assetsToDelete = [];
        $filesToDelete  = [];

        foreach ($allMedia as $media) {
            if (! in_array($media->ref_id, $externalRefIds)) {
                $mediaToDelete[]  = $media->id;                      // Collect media IDs for deletion
                $assetsToDelete[] = $media->media_id;                // Collect asset IDs for deletion
                $filesToDelete[]  = JPATH_ROOT . $media->image_path; // Collect file paths
            }
        }

        // 🔹 Delete from `#__resourcespace_media`
        if (! empty($mediaToDelete)) {
            $query = $db->getQuery(true)
                ->delete($db->quoteName('#__resourcespace_media'))
                ->where($db->quoteName('id') . ' IN (' . implode(',', $mediaToDelete) . ')');
            $db->setQuery($query);
            $db->execute();
        }

        // 🔹 Delete from `#__assets`
        if (! empty($assetsToDelete)) {
            $query = $db->getQuery(true)
                ->delete($db->quoteName('#__assets'))
                ->where($db->quoteName('id') . ' IN (' . implode(',', $assetsToDelete) . ')');
            $db->setQuery($query);
            $db->execute();
        }

        // 🔹 Delete the images from disk
        foreach ($filesToDelete as $filePath) {
            if (file_exists($filePath)) {
                unlink($filePath);
            }
        }
    }

    /**
     * Downloads and saves an image from a URL to Joomla's media folder and adds metadata.
     *
     * @param string $imageUrl The URL of the image to download.
     * @param string $imageName The name to save the image.
     * @param string $refId The reference ID to associate with the image.
     * @param string $extension The extension of the image file.
     *
     * @return string|false The path to the saved image, or false if there was an error.
     */
    public static function saveImageFromUrl($imageUrl, $imageName, $refId, $extension)
    {
                                                            // Get the Joomla media folder path
        $mediaPath = JPATH_ROOT . '/images/resourcespace/'; // Adjust this based on where you want to save it

        // Ensure the media folder exists
        if (! JFolder::exists($mediaPath)) {
            JFolder::create($mediaPath);
        }

        // Full path to save the image
        $savePath = $mediaPath . $imageName . '.' . $extension;

        // Download the image using Joomla's HTTP class
        $http     = JHttpFactory::getHttp();
        $response = $http->get($imageUrl);

        // Check if the request was successful
        if ($response->code == 200) {
            // Save the image content to the media folder
            file_put_contents($savePath, $response->body);

            // Now we insert the metadata into Joomla's media table
            $db = JFactory::getDbo();

            $query = $db->getQuery(true)
                ->select('*')
                ->from($db->quoteName('#__resourcespace_media'))
                ->where($db->quoteName('ref_id') . ' = ' . $db->quote($refId));
            $db->setQuery($query);
            $existingMedia = $db->loadObject();
            // Query for an existing asset by title
            $query2 = $db->getQuery(true)
                ->select('*')
                ->from($db->quoteName('#__assets'))
                ->where($db->quoteName('title') . ' = ' . $db->quote($refId));
            $db->setQuery($query2);

            $existingAsset = $db->loadObject();

            // Prepare the asset metadata
            $asset            = new stdClass();
            $asset->parent_id = 1; // Typically, 1 is the root asset in Joomla
            $asset->title     = $refId;
            $asset->name      = JFilterOutput::stringURLSafe(substr($imageName, 0, 50));
            $asset->path      = '/images/resourcespace/' . substr($imageName, 0, 95) . '.' . $extension; // Path to the saved image
            $asset->ref_id    = $refId;                                                                  // Custom meta field
            $asset->published = 1;                                                                       // Make the asset published
            $asset->level     = 1;
            $asset->rules     = '{}';

            if ($existingAsset) {
                                                 // Update the existing asset
                $asset->id = $existingAsset->id; // Set the ID to update the existing record
                $db->updateObject('#__assets', $asset, 'id');
            } else {
                // Insert a new asset
                $db->insertObject('#__assets', $asset);
                $asset->id = $db->insertid();
            }

            // Prepare the media metadata
            $mediaData              = new stdClass();
            $mediaData->ref_id      = $refId;
            $mediaData->image_name  = $imageName;
            $mediaData->image_path  = '/images/resourcespace/' . $imageName . '.' . $extension;
            $mediaData->media_id    = $existingAsset ? $existingAsset->id : $asset->id; // Link to assets table
            $mediaData->modified_at = date('Y-m-d H:i:s');                              // Update timestamp

            if ($existingMedia) {
                $mediaData->id = $existingMedia->id;
                $db->updateObject('#__resourcespace_media', $mediaData, 'id');
            } else {
                $mediaData->created_at = date('Y-m-d H:i:s'); // Set created timestamp
                $db->insertObject('#__resourcespace_media', $mediaData);
            }

            // Return the path to the saved image
            return '/images/resourcespace/' . $imageName . '.' . $extension;
        } else {
            // Return false if the image could not be downloaded
            return false;
        }
    }

}
